module ultrasonic_sensor(
                input clk,
                input reset,
                input logic [31:0]  chirp_timeout_data, //set by sw;
represents whether to pulse or not.how long to wait for echo before
timeout.
                input logic        write, // signal that device driver
sends when it wants to write something into writedata
                input logic        chipselect, // whether to read chips
or not
                input logic     SENSOR_ECHO,
                output reg      SENSOR_TRIGGER,
                output logic [31:0] status); // 0'd out at first


        logic chirp;
        logic [15:0] timeout;

        always_ff @(posedge clk or posedge reset) begin
                if (reset) begin
                        chirp   <= 1'b0;
                        timeout <= 16'd0;
                end else if (chipselect & write) begin
                        chirp   <= chirp_timeout_data[0];
                        timeout <= chirp_timeout_data[31:16];
                end
        end


    sensor sensor(.clk(clk), .reset(reset), .SENSOR_ECHO(SENSOR_ECHO),
.chirp(chirp),
                          .timeout_data(timeout),
.SENSOR_TRIGGER(SENSOR_TRIGGER), .status(status));
endmodule


module sensor(
        input clk,
        input reset,
        input SENSOR_ECHO,
        input logic chirp,
        input logic [15:0] timeout_data,
        output reg SENSOR_TRIGGER,
        output reg [31:0] status
);


reg [31:0] trigger_counter, timeout_counter, echo_count, timeout_limit;
enum logic [4:0] {init, chirp_hi, wait_echo, count_echo, ready} state;
assign timeout_limit = timeout_data * 1024;
always_ff @(posedge clk) begin
        if (reset) state <= init;
        else case (state)
                init: begin
                        status <= 32'd0;
                        trigger_counter <= 0;
                        if (chirp) state <= chirp_hi;
                      end
                chirp_hi: begin
                        status <= 32'd1;
                        SENSOR_TRIGGER <= 1'd1;
                        timeout_counter <= 0;
                        trigger_counter <= trigger_counter + 1;
                        if (trigger_counter > 500) state <= wait_echo;
                      end
                wait_echo: begin
                        status <= 32'd2;
                        SENSOR_TRIGGER <= 1'd0;
                        timeout_counter <= timeout_counter + 1;
                        echo_count <= 0;
                        if (timeout_counter >= timeout_limit) begin
state <= ready;
                        end
                        else if (SENSOR_ECHO) begin state <=
count_echo;
                        end
                      end
                count_echo: begin
                        status <= 32'd3;
                        echo_count <= echo_count + 1;
                        if (!SENSOR_ECHO || echo_count >=
timeout_limit) state <= ready;
                      end
                ready: begin
                        status <= echo_count;
                        if (!chirp) state <= init;
                      end
                default:                state <= init;
        endcase
end
endmodule